@@ -70,7 +70,7 @@ from types import *
 import re, string 
 import rfc822, mimetools, multifile, StringIO
 from Products.ZWiki.Regexps import wikiname1,wikiname2,bracketedexpr
-from Products.ZWiki.Utils import DLOG
+from Products.ZWiki.Utils import BLATHER
 
 DEFAULTPAGE = 'FrontPage'
 #PAGEINSUBJECTEXP = r'(%s|%s)' % (wikiname1,wikiname2)
@@ -151,7 +151,7 @@ class MailIn:
                  checkrecipient=1,
                  checksubject=1,
                  ):
-        #DLOG('mailin.py: processing incoming message:\n%s' % message)
+        #BLATHER('mailin.py: processing incoming message:\n%s' % message)
 
         # parse the message
         self.context = context
@@ -209,7 +209,7 @@ class MailIn:
                         firstpage.pageWithName(defaultpagename) or firstpage
                     defaultpagename = self.workingpage.title_or_id()
                 else:
-                    DLOG('mailin.py: could not find a working page')
+                    BLATHER('mailin.py: could not find a working page')
                     self.error = '\nSorry, I could not find an existing wiki page to work with.\n\n\n'
                     return
 
@@ -227,6 +227,7 @@ class MailIn:
         accept = getattr(self.folder,'mail_accept_nonmembers',[])
         if not subscribersonly or postingpolicy == 'open':
             pass
+        # XXX poor caching
         elif (self.workingpage.allSubscriptionsFor(self.FromEmail) or
               self.workingpage.allSubscriptionsFor(self.senderEmail)):
             pass
@@ -234,7 +235,7 @@ class MailIn:
             pass
         else:
             self.error = '\nSorry, you must be a subscriber to send mail to this wiki.\n\n\n'
-            DLOG('mailin.py: bounced mail from non-subscriber',
+            BLATHER('mailin.py: bounced mail from non-subscriber',
                  self.FromEmail)
             return
 
@@ -393,7 +394,7 @@ def mailin(self,
         self.REQUEST.set('newtext', body)
         self.REQUEST.set('submitted', 1)
         m.folder.IssueTracker(REQUEST=self.REQUEST)
-        #DLOG('mailin.py: created issue '+subject)
+        #BLATHER('mailin.py: created issue '+subject)
         return 
 
     # a new page ?
@@ -404,7 +405,7 @@ def mailin(self,
         # but "REQUEST has no URL2" and create fails. Leave out for now.
         m.workingpage.create(m.destpagename,text='')
         m.destpage = m.workingpage.pageWithName(m.destpagename)
-        #DLOG('mailin.py: created '+m.destpagename)
+        #BLATHER('mailin.py: created '+m.destpagename)
 
     # add comment
     # use the time of sending, or the time of posting to the wiki - see
@@ -424,4 +425,4 @@ def mailin(self,
                        message_id=m.messageid,
                        in_reply_to=m.inreplyto
                        )
-    #DLOG('mailin.py: commented on '+m.destpagename)
+    #BLATHER('mailin.py: commented on '+m.destpagename)
