@@ -5,8 +5,8 @@ from string import split,join,find,lower,rfind,atoi,strip,lstrip
 from types import *
 
 from TextFormatter import TextFormatter
-from Utils import html_unquote,DLOG,formattedTraceback
-from Defaults import AUTO_UPGRADE
+from Utils import html_unquote,BLATHER,formattedTraceback
+from Defaults import AUTO_UPGRADE, PAGE_METATYPE
 
 class MailSupport:                 
     """
@@ -76,7 +76,7 @@ class MailSupport:
         changed = 0
         f = self.folder().aq_base
 
-        # migrate an old zwiki subscribers attribute
+        # migrate an old zwiki subscribers or wikifornow _subscribers attribute
         oldsubs = None
         if (hasattr(f, 'subscribers') and
             type(f.subscribers) is StringType):
@@ -85,25 +85,20 @@ class MailSupport:
             try:
                 del f.subscribers
             except KeyError:
-                DLOG('failed to delete self.folder().subscribers')
+                BLATHER('failed to delete self.folder().subscribers')
             changed = 1
-
-        # migrate a wikifornow _subscribers attribute
-        oldsubs = None
-        if hasattr(f, '_subscribers'):
+        elif hasattr(f, '_subscribers'):
             oldsubs = f._subscribers.keys()
             try:
                 del f._subscribers
             except KeyError:
-                DLOG('failed to delete self.folder()._subscribers')
+                BLATHER('failed to delete self.folder()._subscribers')
             changed = 1
-        if not hasattr(f, 'subscriber_list'):
-            f.subscriber_list = []
-        # copy old subscribers to subscriber_list, unless it's already
-        # got some
-        # XXX merge instead
-        if oldsubs and not f.subscriber_list:
-            f.subscriber_list = oldsubs
+        # ensure a subscriber_list attribute
+        if not hasattr(f, 'subscriber_list'): f.subscriber_list = []
+        # transfer old subscribers to subscriber_list, unless it's already
+        # populated in which case discard them
+        if oldsubs and not f.subscriber_list: f.subscriber_list = oldsubs
 
         # update _properties
         props = map(lambda x:x['id'], f._properties)
@@ -116,7 +111,7 @@ class MailSupport:
                 ({'id':'subscriber_list','type':'lines','mode':'w'},)
 
         if changed:
-            DLOG('upgraded %s folder subscriber list' % (f.id))
+            BLATHER('upgraded %s folder subscriber list' % (f.id))
 
         # now do the page..
         changed = 0
@@ -131,7 +126,7 @@ class MailSupport:
             try:
                 del self.subscribers
             except KeyError:
-                DLOG('failed to delete %s.subscribers' % (self.id()))
+                BLATHER('failed to delete %s.subscribers' % (self.id()))
             changed = 1
         # copy old subscribers to subscriber_list, unless it's already
         # got some
@@ -146,7 +141,7 @@ class MailSupport:
             try:
                 del self._subscribers
             except KeyError:
-                DLOG('failed to delete %s._subscribers' % (self.id()))
+                BLATHER('failed to delete %s._subscribers' % (self.id()))
             changed = 1
         if oldsubs and not self.subscriber_list:
             self.subscriber_list = oldsubs
@@ -162,7 +157,7 @@ class MailSupport:
                 ({'id':'subscriber_list','type':'lines','mode':'w'},)
 
         if changed:
-            DLOG('upgraded %s subscriber list' % (self.id()))
+            BLATHER('upgraded %s subscriber list' % (self.id()))
 
     ## page subscription api #############################################
 
@@ -183,19 +178,12 @@ class MailSupport:
 
     def isSubscriber(self,email,parent=0):
         """
-        Is email currently subscribed to this page ?
-        With parent flag, check the parent folder's subscriber list instead.
+        Is this email address or member id subscribed to this page ?
 
-        Now that email may be an email address or a username,
-        we must work a bit harder:
-
-        - for a username, check whether it's subscribed as-is. Don't
-        bother looking up it's email address.
-
-        - for an email address, check whether it's subscribed as-is
-        (case-insensitively), but also look for member username(s) which
-        have that email address and check whether any of those are
-        subscribed.
+        With parent flag, check the parent folder's subscriber list
+        instead.  Note "email" may be either an email address
+        (case-insensitive) or a CMF member id.  We'll accept either, and
+        find subscriptions using either.
         """
         subscriber = email
         if subscriber:
@@ -220,10 +208,11 @@ class MailSupport:
                 subs = self._getSubscribers(parent)
                 subs.append(subscriber)
                 self._setSubscribers(subs,parent)
-
+                if not parent: self.index_object()
         if REQUEST:
             REQUEST.RESPONSE.redirect(
-                REQUEST['URL1']+'/subscribeform?email='+subscriber)
+                REQUEST.get('redirectURL',
+                            REQUEST['URL1']+'/subscribeform?email='+subscriber))
 
     def unsubscribe(self, email, REQUEST=None, parent=0):
         """
@@ -239,10 +228,11 @@ class MailSupport:
                 if string.lower(s) == string.lower(subscriber):
                     sl.remove(s)
             self._setSubscribers(sl,parent)
-
+            if not parent: self.index_object()
         if REQUEST:
             REQUEST.RESPONSE.redirect(
-                REQUEST['URL1']+'/subscribeform?email='+subscriber)
+                REQUEST.get('redirectURL',
+                            REQUEST['URL1']+'/subscribeform?email='+subscriber))
 
     ## folder subscription api ###########################################
 
@@ -287,16 +277,38 @@ class MailSupport:
         """
         Return the ids of all pages to which a subscriber is subscribed
         ('whole_wiki' indicates a wiki subscription).
+
+        XXX catalog case duplicates isSubscriber code
         """
         subscriber = email
-        # simple-minded implementation
         subscriptions = []
+        # subscriber may be an email address or a member id, and
+        # they may be subscribed as either
+        email = self.emailAddressFrom(subscriber)
+        if email: email = string.lower(email)
+        usernames = self.usernamesFrom(subscriber)
+
+        if not (email or usernames):
+            return []
         if self.isWikiSubscriber(subscriber):
             subscriptions.append('whole_wiki')
-        # use catalog ?
-        for id, page in self.folder().objectItems(spec='ZWiki Page'):
-            if page.isSubscriber(subscriber):
-                subscriptions.append(id)
+        # optimization: try to use catalog for memory efficiency..
+        # XXX can we do better - index subscriber_list and search
+        # it directly ?
+        if self.hasCatalogIndexesMetadata(
+            (['meta_type','path'], ['subscriber_list'])):
+            pages = self.pages()
+            for page in pages:
+                for sub in page.subscriber_list:
+                    if not sub: continue
+                    if ((email and (string.lower(sub) == email)) or
+                        (usernames and (sub in usernames))):
+                        subscriptions.append(page.id)
+        else:
+            # poor caching
+            for id, page in self.folder().objectItems(spec=PAGE_METATYPE):
+                if page.isSubscriber(subscriber):
+                    subscriptions.append(id)
         return subscriptions
 
     def otherPageSubscriptionsFor(self, email):
@@ -325,11 +337,17 @@ class MailSupport:
         else:
             return 0
 
+    def autoSubscriptionEnabled(self):
+        if getattr(self,'auto_subscribe',0):
+            return 1
+        else:
+            return 0
+
     def isEmailAddress(self,s):
         """
         True if s looks like an email address.
         """
-        if '@' in s: return 1
+        if s and '@' in s: return 1
         else: return 0
     
     def isUsername(self,s):
@@ -394,7 +412,7 @@ class MailSupport:
         """
         Generate a somewhat unique email message-id based on a DateTime
         """
-        msgid = time.strftime('%Y%m%d%H%M%S%Z@')
+        msgid = time.strftime('%Y%m%d%H%M%S')+time.rfc822()[-5:]+'@'
         if hasattr(self,'REQUEST'):
             msgid += re.sub(r'http://','',self.REQUEST.get('SERVER_URL',''))
         msgid = '<%s>' % msgid
@@ -416,10 +434,13 @@ class MailSupport:
         except ValueError: pass
         if not recipients: return
         try:
-            self.sendMailTo(recipients,text,REQUEST,subjectSuffix,subject,
-                            message_id,in_reply_to)
+            self.sendMailTo(recipients,text,REQUEST,
+                            subjectSuffix=subjectSuffix,
+                            subject=subject,
+                            message_id=message_id,
+                            in_reply_to=in_reply_to)
         except:
-            DLOG('failed to send mail to %s: %s' % (recipients,
+            BLATHER('failed to send mail to %s: %s' % (recipients,
                                                     formattedTraceback()))
             admin = getattr(self.folder(),'mail_admin',None)
             if admin:
@@ -427,16 +448,22 @@ class MailSupport:
                     [admin],text or '(no text)',REQUEST,
                     subjectSuffix='ERROR, subscriber mailout failed')
                 except:
-                    DLOG('failed to send error report to admin: %s' % \
+                    BLATHER('failed to send error report to admin: %s' % \
                          formattedTraceback())
 
+    # XXX refactor
+    # todo: use an authenticated CMF member's email property when appropriate
     def sendMailTo(self, recipients, text, REQUEST, subjectSuffix='',
                    subject='',message_id=None,in_reply_to=None):
         """
         Send mail to the specified recipients.
 
         If a mailhost and mail_from property have been configured,
-        attempt to email text to recipients. 
+        attempt to email text to recipients.
+
+        Does all kinds of careful stuff with properties
+        and other info to provide the best possible headers.
+        Refactor.
         """
         if not self.isMailoutEnabled(): return
 
@@ -447,13 +474,13 @@ class MailSupport:
 
         # gather bits and pieces
         mhost=self.MailHost
-        username = self.zwiki_username_or_ip(REQUEST)
+        username = self.usernameFrom(REQUEST)
         if not message_id:
             message_id = self.messageIdFromTime(self.ZopeTime())
         mail_from = getattr(self.folder(),'mail_from','')
         mail_replyto = getattr(self.folder(),'mail_replyto','')
         replytohdr = mail_replyto or mail_from
-        listid = mail_from
+        listid = mail_from or mail_replyto
         # primary recipient, alternatives:
         # 1. "To: ;" causes messy cc header in replies, while
         # 2. "To: replytohdr" sends a copy back to the wiki which
@@ -473,7 +500,9 @@ class MailSupport:
                 fromhdr = mail_replyto
             else:
                 fromhdr = '%s (%s)' % (mail_replyto,username)
+        # link to message - copied from makeCommentHeading
         pageurl = self.page_url()
+        pageurl += '#msg'+re.sub(r'^<(.*)>$',r'\1',message_id) 
         if getattr(self.folder(),'mail_page_name',1):
             pagename = '[%s] ' % self.title_or_id()
         else:
@@ -481,10 +510,7 @@ class MailSupport:
         signature = getattr(self.folder(),
                             'mail_signature',
                             '--\nforwarded from %s' % pageurl)
-        # if using mailin.py's checkrecipient option you can uncomment
-        # this line to set the page name as Reply-to's real name:
-        #replytohdr = replytohdr + ' (%s)'%self.title_or_id()
-
+                           
         # send message - XXX templatize this
         msg = """\
 From: %s
@@ -530,24 +556,24 @@ List-Help: <%s>
            signature,
            )
 
-        # testing support - messages with subject [test] or originating
+        # testing support: messages with subject [test] or originating
         # from TestPage are not mailed out to subscribers
         if (subject == '[test]' or self.title_or_id() == 'TestPage'):
             # log and drop it
-            DLOG('discarding test mailout:\n%s' % msg)
+            BLATHER('discarding test mailout:\n%s' % msg)
             # I tried redirecting to a test SMTP server but it blocked
-            #DLOG('diverting test mailout to test server:\n%s' % msg)
+            #BLATHER('diverting test mailout to test server:\n%s' % msg)
             #try:
             #    self.TestMailHost.send(msg)
-            #    DLOG('sent mailout to test server')
-            #    DLOG('TestMailHost info:',
+            #    BLATHER('sent mailout to test server')
+            #    BLATHER('TestMailHost info:',
             #         self.TestMailHost.smtp_host,
             #         self.TestMailHost.smtp_port)
             #except:
             #    type, val, tb = sys.exc_info()
             #    err = string.join(
             #        traceback.format_exception(type,val,tb),'')
-            #    DLOG('failed to send mailout to test server:',
+            #    BLATHER('failed to send mailout to test server:',
             #         err,
             #         self.TestMailHost.smtp_host,
             #         self.TestMailHost.smtp_port)
@@ -556,11 +582,46 @@ List-Help: <%s>
             #msg = re.sub(r'(?m)(List-Help:.*$)',
             #             r'\1\nX-No-Archive: yes',
             #             msg)
-            #DLOG('sending mailout:\n%s' % msg)
+            #BLATHER('sending mailout:\n%s' % msg)
             #mhost.send(msg)
         else:
-            DLOG('sending mailout:\n%s' % msg)
+            BLATHER('sending mailout:\n%s' % msg)
             mhost.send(msg)
+        # cc comments to an IRC channel via ciabot (or similar)
+        # sent separately so we can provide the special subject 
+        mail_irc_address = getattr(self.folder(),'mail_irc_address',None)
+        mail_irc_subject = getattr(self.folder(),'mail_irc_subject','')
+        if mail_irc_address:
+            mhost.send("""\
+From: %s
+To: %s
+Subject: %s
+
+%s: %s
+%s
+""" \
+            % (fromhdr,
+               mail_irc_address,
+               mail_irc_subject,
+               username,
+               mailouttext,
+               signature,
+               ))
+            BLATHER("""sending mailout to IRC:
+From: %s
+To: %s
+Subject: %s
+
+%s: %s
+%s
+""" \
+            % (fromhdr,
+               mail_irc_address,
+               mail_irc_subject,
+               username,
+               mailouttext,
+               signature,
+               ))
 
     def formatMailout(self, text):
         """
